/* libiocp before 090313 for kernel 2.6.24 */
#ifndef __LIBIOCP_H
#define __LIBIOCP_H

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/types.h>
#include <string.h>
#include <libaio.h>

struct timespec;
struct iocb;

#if defined(__i386__) /* little endian, 32 bits */
#define PADDED(x, y)    x; unsigned y
#define PADDEDptr(x, y) x; unsigned y
#define PADDEDul(x, y)  unsigned long x; unsigned y
#elif defined(__ia64__) || defined(__x86_64__) || defined(__alpha__)
#define PADDED(x, y)    x, y
#define PADDEDptr(x, y) x
#define PADDEDul(x, y)  unsigned long x
#elif defined(__powerpc64__) /* big endian, 64 bits */
#define PADDED(x, y)    unsigned y; x
#define PADDEDptr(x,y)  x
#define PADDEDul(x, y)  unsigned long x
#elif defined(__PPC__)  /* big endian, 32 bits */
#define PADDED(x, y)    unsigned y; x
#define PADDEDptr(x, y) unsigned y; x
#define PADDEDul(x, y)  unsigned y; unsigned long x
#elif defined(__s390x__) /* big endian, 64 bits */
#define PADDED(x, y)    unsigned y; x
#define PADDEDptr(x,y)  x
#define PADDEDul(x, y)  unsigned long x
#elif defined(__s390__) /* big endian, 32 bits */
#define PADDED(x, y)    unsigned y; x
#define PADDEDptr(x, y) unsigned y; x
#define PADDEDul(x, y)  unsigned y; unsigned long x
#else
#error  endian?
#endif

struct cp_event {
	PADDEDul(result,  __pad1);
	PADDEDptr(void *completion_key, __pad2);
	PADDEDptr(struct iocb *user,  __pad3);
}; /* 32 bytes */

#undef PADDED
#undef PADDEDptr
#undef PADDEDul


#define	SYS_CP_CREATE_PORT		325
#define	SYS_CP_ADD_FILE_TO_PORT		326
#define	SYS_CP_DEL_FILE_FROM_PORT	327
#define	SYS_CP_DESTROY_PORT		328
#define SYS_CP_GET_COMPLETION_STATUS    329
#define SYS_CP_POST_COMPLETION_STATUS   330
#define SYS_AIO_POST_IO_COMPLETION      331

typedef unsigned long io_port_t;

#define cp_create_port(args...) syscall(SYS_CP_CREATE_PORT, ## args)
#define cp_add_file_to_port(args...) syscall(SYS_CP_ADD_FILE_TO_PORT, ## args)
#define cp_del_file_from_port(args...) syscall(SYS_CP_DEL_FILE_FROM_PORT, ## args)
#define cp_destroy_port(args...) syscall(SYS_CP_DESTROY_PORT, ## args)
#define cp_get_completion_status(args...) syscall(SYS_CP_GET_COMPLETION_STATUS, ## args)
#define cp_post_completion_status(args...) syscall(SYS_CP_POST_COMPLETION_STATUS, ## args)
#define aio_post_io_completion(args...) syscall(SYS_AIO_POST_IO_COMPLETION, ## args)


#ifdef __cplusplus
}
#endif

#endif /* __LIBIOCP_H */

